import json
import pandas as pd
# creates a csv file from the fhir schema
# This is used to generate empty mapping files which can be used to 
# map other data sources to FHIR


fhir_json_schema_file = ''# place the path to the fhir json shcmea here, it can be downloaded from here: http://hl7.org/fhir/fhir.schema.json.zip

nsheet = {'source_name_1':[], 'table_name_1':[], 'column_1':[], 'source_name_2':[], 'table_name_2':[], 'column_2':[], 'map_on':[]}
bad_keys = ['base64Binary', 'ResourceList', 'boolean', 'canonical']

def parse_ref(ref_str):
    #"$ref": "#/definitions/id"
    tmp = ref_str.split("/")[-1::][0]
    return tmp

def parse_table(data, resource_key):
    if 'properties' in data['definitions'][resource_key]:
        for k in data['definitions'][resource_key]['properties']:
            ts = data['definitions'][resource_key]['properties'][k]
            nsheet['source_name_1'].append("FHIR")
            nsheet['table_name_1'].append(resource_key)
            nsheet['column_1'].append(k)
            if "items" in ts:
                if "$ref" in ts['items']:
                    nsheet['table_name_2'].append(parse_ref(ts['items']['$ref']))
                    nsheet['source_name_2'].append("FHIR")
                    nsheet['column_2'].append("id")
                else:
                    nsheet['table_name_2'].append("")
                    nsheet['source_name_2'].append("")
                    nsheet['column_2'].append("")
            elif "$ref" in ts:
                nsheet['table_name_2'].append(parse_ref(ts['$ref']))
                nsheet['source_name_2'].append("FHIR")
                nsheet['column_2'].append("id")
            else:
                nsheet['table_name_2'].append("")
                nsheet['source_name_2'].append("")
                nsheet['column_2'].append("")

            if "type" in data['definitions'][resource_key]['properties'][k]:
                if data['definitions'][resource_key]['properties'][k]['type'] == "array":
                    nsheet['map_on'].append("MANY")
                else:
                    nsheet['map_on'].append("EQUALITY")
            else:
                nsheet['map_on'].append("EQUALITY")


with open(fhir_json_schema_file, 'r', encoding="utf8") as json_schema:
    data = json.load(json_schema)
    for k in data['definitions']:
        parse_table(data, k)

t = pd.DataFrame.from_dict(nsheet)
print(t)
t.to_csv("FHIR_schema.csv", index=False)


